<?php

namespace Ebizmarts\MageMonkey\Helper;

class Import extends \Ebizmarts\MageMonkey\Helper\Data {

    /**
     * @param \Ebizmarts\MageMonkey\Helper\Data $helper
     * @param \Magento\Customer\Model\Customer $customer
     * @param \Magento\Customer\Model\Session $customerSession
     */
    protected $_api = null;
    protected $_subscriber = null;
    protected $_loggerImport = null;

    public function __construct(
    \Magento\Framework\App\Helper\Context $context, \Magento\Store\Model\StoreManagerInterface $storeManager, \Ebizmarts\MageMonkey\Model\Logger\Logger $logger, \Magento\Customer\Model\GroupRegistry $groupRegistry, \Magento\Framework\App\Request\Http $request, \Ebizmarts\MageMonkey\Model\Api $api, \Magento\Newsletter\Model\Subscriber $subscriber, \Ebizmarts\MageMonkey\Model\Logger\LoggerImport $loggerImport, \Magento\Customer\Api\CustomerRepositoryInterface $customerInterface
    ) {

        $this->_api = $api;
        $this->_subscriber = $subscriber;
        $this->_loggerImport = $loggerImport;
        parent::__construct($context, $storeManager, $logger, $groupRegistry, $request, $customerInterface);
    }

    public function sendToMailchimp($subscribersIds) {
        $api = $this->_api;
        $status = '';
        $count = 0;

        $countOk = 0;
        $countStatusNotAllowed = 0;
        $countError = 0;
        $this->_loggerImport->info(" #### START REQUEST COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $this->_loggerImport->info("processing subscriber id " . $subscriberId);

            $subscriber = $this->_subscriber->load($subscriberId);

            if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED || $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {

                if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                    $status = 'subscribed';
                } else {
                    $status = 'unsubscribed';
                }

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                $mergeVars = $this->getMergeVars($subscriber,false,true);

                if ($mergeVars) {

                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
                } else {
                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
                }

                try {
                    if ($subscriber->getMagemonkeyId()) {
                                         
                        $this->_loggerImport->info("update subscriber id " . $subscriberId);
                        $return = $api->listUpdateMember($this->getDefaultList($subscriber->getStoreId()), json_encode($data), $subscriber->getMagemonkeyId());
                        
                    } else {
                        $this->_loggerImport->info("new subscriber id " . $subscriberId);
                        $return = $api->listCreateMember($this->getDefaultList($subscriber->getStoreId()), json_encode($data));

                        if (isset($return->id)) {
                            $subscriber->setMagemonkeyId($return->id);
                            $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp id " . $return->id);
                        } else {
                            $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp none");
                        }
                    }

                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            } else {
                $this->_loggerImport->info("status not allowed for subscriber id" . $subscriberId);
                $countStatusNotAllowed++;
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " / Result NotAllowed: " . $countStatusNotAllowed . " #### ");

        return $count;
    }

    public function sendToMailchimpOnlyNew($subscribersIds) {
        $api = $this->_api;
        $status = '';
        $count = 0;

        $countOk = 0;
        $countError = 0;

        $this->_loggerImport->info(" #### START REQUEST ONLY NEW COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {
            
            $subscriber = $this->_subscriber->load($subscriberId);

            if (!$subscriber->getMagemonkeyId() && ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED || $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED)) {

                $this->_loggerImport->info("processing subscriber id " . $subscriberId);
                
                if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                    $status = 'subscribed';
                } else {
                    $status = 'unsubscribed';
                }

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                $mergeVars = $this->getMergeVars($subscriber,false,true);

                if ($mergeVars) {
                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
                } else {
                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
                }

                try {

                    $this->_loggerImport->info("new subscriber id " . $subscriberId);
                    $return = $api->listCreateMember($this->getDefaultList($subscriber->getStoreId()), json_encode($data));

                    if (isset($return->id)) {
                        $subscriber->setMagemonkeyId($return->id);
                        $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp id " . $return->id);
                    } else {
                        $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp none");
                    }


                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " #### ");

        return $count;
    }

}
